#ifndef SLAM_DECISION_H_
#define SLAM_DECISION_H_

#include <iostream>
#include <vector>

enum RoadType:uint8_t {
    NOT_AVAILABLE = 0,
    RAMP_SPLIT = 1,               // "匝道内分岔路"
    HIGHWAY_CONNECTOR = 2,        // "高速连接路" 
    HIGHWAY_GROUND_CONNECTOR = 3, // "高速地面连接路" 
    MAIN_ROAD_FORK = 4,           // "主路分岔路" 
    RAMP_MERGING_MAIN_ROAD = 5,   // "匝道汇合主路" 
    MAIN_ROAD_MERGING_ROAD = 6,   // "主路汇合路" 
    VIRTUAL_MAIN_ROAD = 7,        // "虚拟主路"
	RMD_SPLIT_ROAD = 8,           // "提醒用主路匝道分岔路，主路径为主路"
	RMD_MERGE_ROAD = 9            // "提醒用匝道汇合主路，主路径为主路"
};

struct AdjacentExtendLaneStruct {
    uint8_t adjacent_extendlane_id;           // 车道id (Lane ID)
    float adjacent_extendlane_offset;         // 相邻段的起始位置与自车距离 (Starting position offset of the adjacent section from the vehicle)
    float adjacent_extendlane_endoffset;      // 相邻段的终止位置与自车距离 (Ending position offset of the adjacent section from the vehicle)
};

struct SMAStruct {
    uint8_t lane_id;
    uint8_t type;
    uint8_t sub_type;
    float start_offset;
    float end_offset;
};

struct WPExtendLaneStruct {
    uint8_t lane_id;                                     // 车道id
    uint8_t padding[3];  
    float lane_latest_lc_point_offset_to_wp;                      // 车道最晚变道点
    std::vector<AdjacentExtendLaneStruct> left_extend_lane;            // 左邻车道列表
    std::vector<AdjacentExtendLaneStruct> right_extend_lane;           // 右邻车道列表
    uint8_t lane_priority;        // 可不赋值
    uint8_t is_emergency_lane;    // 是否不可行驶车道，如应急车道
    float current_extendlane_offset;     // 车道起始位置
    float current_extendlane_endoffset;  // 车道终止位置
    std::vector<SMAStruct> sma_info;     // 暂不赋值
};


struct TransitLaneStruct {
    uint8_t transit_type;          // 车道过渡类型 (Lane Transition Type)
    uint8_t transit_lane_id;       // 车道id (Lane ID)
    uint8_t padding[2]; 
    float transit_lane_offset;     // 车道过渡点与自车距离 (Distance from the vehicle to the lane transition point)
    uint8_t transit_connected_lane_id; // 连接车道id
};


// struct RemindWayPointStruct {
//     uint8_t reminder_waypoint_target_road_type;               // 提醒用航点类型
//     uint32_t reminder_waypoint_id;                          // 提醒用航点ID
//     std::vector<uint8_t> reminder_waypoint_lane_id_list;                    // 提醒用航点目标车道
//     float reminder_waypoint_offset;                        // 提醒用航点距离，单位: 米
//     std::vector<WPExtendLaneStruct> reminder_wp_extendlane_list;          // 提醒用航点相关extend lane列表
//     uint8_t reminder_waypoint_speed_limit;      // 暂不赋值
// };


// struct ControlWayPointStruct {
//     RoadType control_waypoint_target_road_type;            // 控制用航点类型
//     uint32_t control_waypoint_id;                          // 控制用航点ID
//     std::vector<uint8_t> control_waypoint_lane_id_list;    // 控制用航点目标车道
//     std::vector<uint8_t> control_waypoint_raw_lane_id_list;// 控制用航点原始目标车道
//     float control_waypoint_offset;                         // 控制用航点与自车距离，单位: 米
//     std::vector<WPExtendLaneStruct> control_wp_extendlane_list;     // 控制用航点相关extend lane列表
//     uint8_t deceleration_lane_type;              // 暂不赋值
//     uint8_t control_waypoint_speed_limit;        // 暂不赋值
// };

struct MsgHeader {
    uint64_t time_stamp;     // 时间戳 (Timestamp)
    uint16_t cycle_counter;  // 循环计数 (Cycle Counter) - Used for checking input provider node status
    uint8_t padding[2]; 
};

struct ControlWaypoints {
    MsgHeader msg_header;                                // 信号头 (Message Header)
    std::vector<ControlWayPointStruct> control_waypoints_list;     // 控制用航点列表 (List of Control Way Points)
    std::vector<RemindWayPointStruct> reminder_waypoint;         // 提醒用航点 (Reminder Way Point) - Optional
    std::vector<TransitLaneStruct> transit_lanes_list;   // 可暂不提供 (Transit Lane List) - Optional
    float remain_distance_major_path;                    // 主路径剩余有效地图数据长度 (Remaining Distance of Major Path) - Optional
};


#endif //SLAM_DECISION_H_